from __future__ import absolute_import, division, print_function

import stripe


class TestUsageRecordSummary(object):
    def test_is_listable(self, request_mock):
        resource = stripe.SubscriptionItem.retrieve('si_123')
        usage_record_summaries = resource.usage_record_summaries()
        request_mock.assert_requested(
            'get',
            '/v1/subscription_items/si_123/usage_record_summaries'
        )
        assert isinstance(usage_record_summaries.data, list)
        assert isinstance(usage_record_summaries.data[0],
                          stripe.UsageRecordSummary)



from __future__ import absolute_import, division, print_function

import stripe


class TestListableAPIResource(object):
    class MyListable(stripe.api_resources.abstract.ListableAPIResource):
        OBJECT_NAME = 'mylistable'

    def test_all(self, request_mock):
        request_mock.stub_request(
            'get',
            '/v1/mylistables',
            {
                'object': 'list',
                'data': [
                    {
                        'object': 'charge',
                        'name': 'jose',
                    },
                    {
                        'object': 'charge',
                        'name': 'curly',
                    }
                ],
                'url': '/v1/charges',
                'has_more': False,
            },
            rheaders={'request-id': 'req_id'}
        )

        res = self.MyListable.list()
        request_mock.assert_requested(
            'get',
            '/v1/mylistables',
            {}
        )
        assert len(res.data) == 2
        assert all(isinstance(obj, stripe.Charge) for obj in res.data)
        assert res.data[0].name == 'jose'
        assert res.data[1].name == 'curly'

        assert res.last_response is not None
        assert res.last_response.request_id == 'req_id'



from __future__ import absolute_import, division, print_function

import stripe


class TestSingletonAPIResource(object):
    class MySingleton(stripe.api_resources.abstract.SingletonAPIResource):
        OBJECT_NAME = 'mysingleton'

    def test_retrieve(self, request_mock):
        request_mock.stub_request(
            'get',
            '/v1/mysingleton',
            {
                'single': 'ton'
            },
            rheaders={'request-id': 'req_id'}
        )

        res = self.MySingleton.retrieve()

        request_mock.assert_requested(
            'get',
            '/v1/mysingleton',
            {}
        )
        assert res.single == 'ton'

        assert res.last_response is not None
        assert res.last_response.request_id == 'req_id'



from __future__ import absolute_import, division, print_function

from stripe.api_resources.abstract import CreateableAPIResource
from stripe.api_resources.abstract import UpdateableAPIResource
from stripe.api_resources.abstract import ListableAPIResource


class FileLink(CreateableAPIResource, ListableAPIResource,
               UpdateableAPIResource):
    OBJECT_NAME = 'file_link'



from __future__ import absolute_import, division, print_function

import stripe


TEST_RESOURCE_ID = 'cus_123'
TEST_SUB_ID = 'sub_123'
TEST_SOURCE_ID = 'ba_123'


class TestCustomer(object):
    def test_is_listable(self, request_mock):
        resources = stripe.Customer.list()
        request_mock.assert_requested(
            'get',
            '/v1/customers'
        )
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], stripe.Customer)

    def test_is_retrievable(self, request_mock):
        resource = stripe.Customer.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            'get',
            '/v1/customers/%s' % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.Customer)

    def test_is_creatable(self, request_mock):
        resource = stripe.Customer.create()
        request_mock.assert_requested(
            'post',
            '/v1/customers'
        )
        assert isinstance(resource, stripe.Customer)

    def test_is_saveable(self, request_mock):
        resource = stripe.Customer.retrieve(TEST_RESOURCE_ID)
        resource.metadata['key'] = 'value'
        resource.save()
        request_mock.assert_requested(
            'post',
            '/v1/customers/%s' % TEST_RESOURCE_ID
        )

    def test_is_modifiable(self, request_mock):
        resource = stripe.Customer.modify(
            TEST_RESOURCE_ID,
            metadata={'key': 'value'}
        )
        request_mock.assert_requested(
            'post',
            '/v1/customers/%s' % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.Customer)

    def test_is_deletable(self, request_mock):
        resource = stripe.Customer.retrieve(TEST_RESOURCE_ID)
        resource.delete()
        request_mock.assert_requested(
            'delete',
            '/v1/customers/%s' % TEST_RESOURCE_ID
        )
        assert resource.deleted is True

    def test_can_delete_discount(self, request_mock):
        resource = stripe.Customer.retrieve(TEST_RESOURCE_ID)
        resource.delete_discount()
        request_mock.assert_requested(
            'delete',
            '/v1/customers/%s/discount' % TEST_RESOURCE_ID
        )


class TestCustomerSources(object):
    def test_is_creatable(self, request_mock):
        stripe.Customer.create_source(
            TEST_RESOURCE_ID,
            source='btok_123'
        )
        request_mock.assert_requested(
            'post',
            '/v1/customers/%s/sources' % TEST_RESOURCE_ID
        )

    def test_is_retrievable(self, request_mock):
        stripe.Customer.retrieve_source(
            TEST_RESOURCE_ID,
            TEST_SOURCE_ID
        )
        request_mock.assert_requested(
            'get',
            '/v1/customers/%s/sources/%s' % (TEST_RESOURCE_ID,
                                             TEST_SOURCE_ID)
        )

    def test_is_modifiable(self, request_mock):
        stripe.Customer.modify_source(
            TEST_RESOURCE_ID,
            TEST_SOURCE_ID,
            metadata={'foo': 'bar'}
        )
        request_mock.assert_requested(
            'post',
            '/v1/customers/%s/sources/%s' % (TEST_RESOURCE_ID,
                                             TEST_SOURCE_ID)
        )

    def test_is_deletable(self, request_mock):
        stripe.Customer.delete_source(
            TEST_RESOURCE_ID,
            TEST_SOURCE_ID
        )
        request_mock.assert_requested(
            'delete',
            '/v1/customers/%s/sources/%s' % (TEST_RESOURCE_ID,
                                             TEST_SOURCE_ID)
        )

    def test_is_listable(self, request_mock):
        resources = stripe.Customer.list_sources(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            'get',
            '/v1/customers/%s/sources' % TEST_RESOURCE_ID
        )
        assert isinstance(resources.data, list)



from __future__ import absolute_import, division, print_function

from six.moves.urllib.parse import parse_qs, urlparse

import stripe


class TestOAuth(object):
    def test_authorize_url(self, request_mock):
        url = stripe.OAuth.authorize_url(
            scope='read_write',
            state='csrf_token',
            stripe_user={
                'email': 'test@example.com',
                'url': 'https://example.com/profile/test',
                'country': 'US',
            })

        o = urlparse(url)
        params = parse_qs(o.query)

        assert o.scheme == 'https'
        assert o.netloc == 'connect.stripe.com'
        assert o.path == '/oauth/authorize'

        assert params['client_id'] == ['ca_123']
        assert params['scope'] == ['read_write']
        assert params['stripe_user[email]'] == ['test@example.com']
        assert params['stripe_user[url]'] == \
            ['https://example.com/profile/test']
        assert params['stripe_user[country]'] == ['US']

    def test_token(self, request_mock):
        request_mock.stub_request(
            'post',
            '/oauth/token',
            {
                'access_token': 'sk_access_token',
                'scope': 'read_only',
                'livemode': 'false',
                'token_type': 'bearer',
                'refresh_token': 'sk_refresh_token',
                'stripe_user_id': 'acct_test',
                'stripe_publishable_key': 'pk_test',
            }
        )

        resp = stripe.OAuth.token(
            grant_type='authorization_code',
            code='this_is_an_authorization_code',
        )
        request_mock.assert_requested(
            'post',
            '/oauth/token',
            {
                'grant_type': 'authorization_code',
                'code': 'this_is_an_authorization_code',
            }
        )
        assert resp['access_token'] == 'sk_access_token'

    def test_deauthorize(self, request_mock):
        request_mock.stub_request(
            'post',
            '/oauth/deauthorize',
            {
                'stripe_user_id': 'acct_test_deauth',
            }
        )

        resp = stripe.OAuth.deauthorize(stripe_user_id='acct_test_deauth')
        request_mock.assert_requested(
            'post',
            '/oauth/deauthorize',
            {
                'client_id': 'ca_123',
                'stripe_user_id': 'acct_test_deauth',
            }
        )
        assert resp['stripe_user_id'] == 'acct_test_deauth'



from __future__ import absolute_import, division, print_function

import pytest

import stripe


TEST_RESOURCE_ID = 'src_123'


class TestSource(object):
    def test_is_retrievable(self, request_mock):
        resource = stripe.Source.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            'get',
            '/v1/sources/%s' % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.Source)

    def test_is_creatable(self, request_mock):
        resource = stripe.Source.create(
            type='card',
            token='tok_123'
        )
        request_mock.assert_requested(
            'post',
            '/v1/sources'
        )
        assert isinstance(resource, stripe.Source)

    def test_is_saveable(self, request_mock):
        resource = stripe.Source.retrieve(TEST_RESOURCE_ID)
        resource.metadata['key'] = 'value'
        resource.save()
        request_mock.assert_requested(
            'post',
            '/v1/sources/%s' % TEST_RESOURCE_ID
        )

    def test_is_modifiable(self, request_mock):
        resource = stripe.Source.modify(
            TEST_RESOURCE_ID,
            metadata={'key': 'value'}
        )
        request_mock.assert_requested(
            'post',
            '/v1/sources/%s' % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.Source)

    def test_is_detachable_when_attached(self, request_mock):
        resource = stripe.Source.construct_from({
            'id': TEST_RESOURCE_ID,
            'object': 'source',
            'customer': 'cus_123'
        }, stripe.api_key)
        source = resource.detach()
        assert source is resource
        request_mock.assert_requested(
            'delete',
            '/v1/customers/cus_123/sources/%s' % TEST_RESOURCE_ID
        )

    def test_is_not_detachable_when_unattached(self):
        resource = stripe.Source.retrieve(TEST_RESOURCE_ID)
        with pytest.raises(stripe.error.InvalidRequestError):
            resource.detach()

    def test_is_verifiable(self, request_mock):
        resource = stripe.Source.retrieve(TEST_RESOURCE_ID)
        source = resource.verify(values=[1, 2])
        assert source is resource
        request_mock.assert_requested(
            'post',
            '/v1/sources/%s/verify' % TEST_RESOURCE_ID,
            {'values': [1, 2]}
        )



from __future__ import absolute_import, division, print_function

import stripe


TEST_RESOURCE_ID = 'frr_123'


class TestReportRun(object):
    def test_is_creatable(self, request_mock):
        resource = stripe.reporting.ReportRun.create(
            parameters={
                'connected_account': 'acct_123',
            },
            report_type='activity.summary.1',
        )
        request_mock.assert_requested(
            'post',
            '/v1/reporting/report_runs'
        )
        assert isinstance(resource, stripe.reporting.ReportRun)

    def test_is_listable(self, request_mock):
        resources = stripe.reporting.ReportRun.list()
        request_mock.assert_requested(
            'get',
            '/v1/reporting/report_runs'
        )
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], stripe.reporting.ReportRun)

    def test_is_retrievable(self, request_mock):
        resource = stripe.reporting.ReportRun.retrieve(TEST_RESOURCE_ID)
        request_mock.assert_requested(
            'get',
            '/v1/reporting/report_runs/%s' % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.reporting.ReportRun)
